//////////////////////////////////////////////////////////////////////////
//
//  ct_windows.h    window pairs, globals (window handles etc.) 
//
//////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <windows.h>   // Win32 APIs
#include <wincon.h>    // Windows console

#include "ct_windows.h"

// GLOBALS: window handles. Used by graphics and mouse functions. Hide them later.

HWND hConsoleWindowHandle;
HWND hGraphicWindowHandle;

// GLOBALS: mouse state. Retrieved by routines from ct_mouse

// enum BUTTON_STATE { BUTTON_UP, BUTTON_DOWN } declared in ct_windows.h
BUTTON_STATE buttonState = BUTTON_UP;

int   xMousePos = 0;    // mouse position
int   yMousePos = 0; 

// GLOBALS: for graphics

HPEN      hCurrentPen;
HBRUSH    hCurrentBrush;
int       currentPenPointX = 0;
int       currentPenPointY = 0;

// declared in ct_windows.h
// const int GRAPHICS_WINDOW_WIDTH = 400;
// const int GRAPHICS_WINDOW_HEIGHT = 400;

// WindowPair : create a graphics windows and its thread with message loop

void WindowPair(){

   DWORD thread_id;
   HANDLE thread_handle = CreateThread( NULL, 
                                        0, 
                                        GraphicWindowThreadFunc, 
                                        NULL , 
                                        0, 
                                        &thread_id ); 

   HWND hCW = GetConsoleHwnd();
   hConsoleWindowHandle = hCW;

   // console window new position and size

   if( BOTTOM_ALIGNED ){
	   MoveWindow( hCW, 
		   GRAPHICS_WINDOW_OFFSET_X,
		   GRAPHICS_WINDOW_OFFSET_Y + GRAPHICS_WINDOW_HEIGHT + TITLE_BAR_HEIGHT,
		   CONSOLE_WINDOW_WIDTH,
		   CONSOLE_WINDOW_HEIGHT, 
		   true );
   }
   else{
	   MoveWindow( hCW, 
		   GRAPHICS_WINDOW_OFFSET_X + GRAPHICS_WINDOW_WIDTH + WINDOW_BORDER_WIDTH,
		   GRAPHICS_WINDOW_OFFSET_Y,
		   CONSOLE_WINDOW_WIDTH,
		   CONSOLE_WINDOW_HEIGHT, 
		   true );
   }
		
   
   // make the console the top foreground window -- is this BAD, too drastic?
   SetForegroundWindow( hCW ); 
}

DWORD WINAPI GraphicWindowThreadFunc( LPVOID arg){
  
   WNDCLASS WindowClass;      // Structure to hold our window's attributes

   static char szAppName[] = "Ugly Hack"; // Define window class name
   HWND hWnd;                             // Window handle
   MSG msg;                               // Windows message structure

   // Redraw the window if the size changes
   WindowClass.style   = CS_HREDRAW | CS_VREDRAW | CS_NOCLOSE ;

   // Define our procedure for message handling 
   WindowClass.lpfnWndProc = WindowProc;

   WindowClass.cbClsExtra = 0;    // No extra bytes after the window class
   WindowClass.cbWndExtra = 0;    // structure or the window instance

   WindowClass.hInstance = GetModuleHandle( NULL );  // Application instance handle

   // Set default application icon
   WindowClass.hIcon = LoadIcon(0, IDI_APPLICATION);

   // Set window cursor to be the standard arrow
   WindowClass.hCursor = LoadCursor(0, IDC_ARROW);

   // Set gray brush for background color
   WindowClass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);

   WindowClass.lpszMenuName = 0;  // No menu, so no menu resource name

   WindowClass.lpszClassName = szAppName;  // Set class name

   // Now register our window class
   RegisterClass(&WindowClass);

   //  Now we can create the window
   hWnd = CreateWindow(
          szAppName,                      // the window class name
          "UgLY hAcK GRapHics wInDOw",    // The window title
          WS_DLGFRAME,                    // Window style as overlapped
          GRAPHICS_WINDOW_OFFSET_X,       // Default screen position of upper left
          GRAPHICS_WINDOW_OFFSET_Y,       // corner of our window as x,y...
          GRAPHICS_WINDOW_WIDTH + WINDOW_BORDER_WIDTH,  // Default window size
          GRAPHICS_WINDOW_HEIGHT + TITLE_BAR_HEIGHT,    // with corrections 
          0,                              // No parent window
          0,                              // No menu
          GetModuleHandle( NULL ),        // Program Instance handle
          0                               // No window creation data
          );

   ShowWindow(hWnd, SW_SHOWNORMAL);       // Display the window

   UpdateWindow(hWnd);           // Cause window client area to be drawn

   // set GLOBALS now

   hGraphicWindowHandle = hWnd;

   // set the GLOBAL pen and brush

   hCurrentPen   = (HPEN) GetStockObject( BLACK_PEN );
   hCurrentBrush = (HBRUSH) GetStockObject(BLACK_BRUSH);

   // The message loop
   while(GetMessage(&msg, hWnd, 0, 0) == TRUE)   // Get any messages
   {
      // from here one can print message contents to cout of
      // the console window. 
        
      TranslateMessage(&msg);                 // Translate the message
      DispatchMessage(&msg);                  // Dispatch the message
   }

   return msg.wParam;                      // End so return to Windows
}



LRESULT APIENTRY WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{ 
    switch (uMsg) { 

        case WM_CREATE: 
 
            return 0; 
 
	    //case WM_PAINT: 
	    //case WM_SIZE: 
	    //case WM_MOVE:

	case WM_LBUTTONDOWN: 

	    buttonState = BUTTON_DOWN;       
	    break;

	    //case WM_MOUSEMOVE: 

	case WM_LBUTTONUP: 
	  
	    buttonState = BUTTON_UP;       
	    break;

    case WM_MOUSEMOVE: 
        // fwKeys = wParam;        // key flags 
        xMousePos = LOWORD(lParam);  // horizontal position of cursor 
        yMousePos = HIWORD(lParam);  // vertical position of cursor 
        break; 

 	case WM_DESTROY: 

        PostQuitMessage(0); 
        break; 
 
    default: 
            return DefWindowProc(hwnd, uMsg, wParam, lParam); 
    } 
    return (LRESULT) NULL; 
} 

// Retrieve console window handle, needed for moving and resizing
// This is from MSoft KB.

HWND GetConsoleHwnd(void)
{
  #define MY_BUFSIZE 1024             // Buffer size for console window titles.
  HWND hwndFound;                     // This is what is returned to the caller.
  char pszNewWindowTitle[MY_BUFSIZE]; // Contains fabricated
                                      // WindowTitle.
  char pszOldWindowTitle[MY_BUFSIZE]; // Contains original
                                      // WindowTitle.

  // Fetch current window title.

  GetConsoleTitle(pszOldWindowTitle, MY_BUFSIZE);

  // Format a "unique" NewWindowTitle.

  wsprintf(pszNewWindowTitle,"%d/%d",
               GetTickCount(),
            GetCurrentProcessId());

  // Change current window title.

  SetConsoleTitle(pszNewWindowTitle);

  // Ensure window title has been updated.

  Sleep(40);

  // Look for NewWindowTitle.
  hwndFound=FindWindow(NULL, pszNewWindowTitle);

  // Restore original window title.

  SetConsoleTitle(pszOldWindowTitle);

  return(hwndFound);
}





